@@ -6,9 +6,11 @@
  */
 #include "appmonitor.h"
 #include <algorithm>
-#include "fcitx/misc_p.h"
-#include "display.h"
-#include "plasmaappmonitor.h"
+#include <memory>
+#include <optional>
+#include <string>
+#include <unordered_map>
+#include <utility>
 
 namespace fcitx {
 
