@@ -6,15 +6,39 @@
  */
 #include "waylandimserver.h"
 #include <sys/mman.h>
+#include <sys/types.h>
+#include <unistd.h>
 #include <algorithm>
+#include <cstddef>
+#include <cstdint>
+#include <cstring>
 #include <memory>
+#include <string>
+#include <string_view>
+#include <tuple>
+#include <utility>
+#include <xkbcommon/xkbcommon.h>
+#include "fcitx-utils/capabilityflags.h"
+#include "fcitx-utils/event.h"
+#include "fcitx-utils/eventloopinterface.h"
+#include "fcitx-utils/key.h"
+#include "fcitx-utils/keysym.h"
 #include "fcitx-utils/macros.h"
+#include "fcitx-utils/textformatflags.h"
 #include "fcitx-utils/utf8.h"
+#include "fcitx/event.h"
+#include "fcitx/focusgroup.h"
+#include "fcitx/inputcontext.h"
+#include "fcitx/inputcontextmanager.h"
+#include "fcitx/instance.h"
 #include "virtualinputcontext.h"
 #include "wayland-text-input-unstable-v1-client-protocol.h"
 #include "wayland_public.h"
 #include "waylandim.h"
+#include "waylandimserverbase.h"
 #include "wl_seat.h"
+#include "zwp_input_method_context_v1.h"
+#include "zwp_input_method_v1.h"
 
 #ifdef __linux__
 #include <linux/input-event-codes.h>
