@@ -6,14 +6,36 @@
  */
 #include "waylandimserverv2.h"
 #include <sys/mman.h>
-#include "fcitx-utils/keysymgen.h"
+#include <sys/types.h>
+#include <algorithm>
+#include <cstdint>
+#include <cstring>
+#include <ctime>
+#include <memory>
+#include <string>
+#include <tuple>
+#include <utility>
+#include <xkbcommon/xkbcommon.h>
+#include "fcitx-utils/capabilityflags.h"
+#include "fcitx-utils/event.h"
+#include "fcitx-utils/eventloopinterface.h"
+#include "fcitx-utils/key.h"
+#include "fcitx-utils/keysym.h"
+#include "fcitx-utils/macros.h"
+#include "fcitx-utils/textformatflags.h"
 #include "fcitx-utils/unixfd.h"
 #include "fcitx-utils/utf8.h"
+#include "fcitx/event.h"
 #include "fcitx/inputcontext.h"
+#include "fcitx/instance.h"
 #include "virtualinputcontext.h"
 #include "wayland-text-input-unstable-v3-client-protocol.h"
+#include "wayland_public.h"
 #include "waylandim.h"
+#include "waylandimserverbase.h"
 #include "wl_seat.h"
+#include "zwp_input_method_manager_v2.h"
+#include "zwp_virtual_keyboard_manager_v1.h"
 
 namespace fcitx {
 
@@ -99,9 +121,8 @@ void WaylandIMServerV2::refreshSeat() {
         if (icMap_.count(seat.get())) {
             continue;
         }
-        auto *ic = new WaylandIMInputContextV2(
-            inputContextManager(), this, seat,
-            virtualKeyboardManagerV1_->createVirtualKeyboard(seat.get()));
+        auto *ic =
+            new WaylandIMInputContextV2(inputContextManager(), this, seat);
         ic->setFocusGroup(group_);
         ic->setCapabilityFlags(baseFlags);
     }
@@ -121,11 +142,14 @@ void WaylandIMServerV2::remove(wayland::WlSeat *seat) {
 
 WaylandIMInputContextV2::WaylandIMInputContextV2(
     InputContextManager &inputContextManager, WaylandIMServerV2 *server,
-    std::shared_ptr<wayland::WlSeat> seat, wayland::ZwpVirtualKeyboardV1 *vk)
+    std::shared_ptr<wayland::WlSeat> seat)
     : VirtualInputContextGlue(inputContextManager), server_(server),
       seat_(std::move(seat)),
-      ic_(server->inputMethodManagerV2()->getInputMethod(seat_.get())),
-      vk_(vk) {
+      ic_(server->inputMethodManagerV2()->getInputMethod(seat_.get())) {
+    if (server_->parent()->persistentVirtualKeyboard()) {
+        vk_.reset(server_->virtualKeyboardManagerV1()->createVirtualKeyboard(
+            seat_.get()));
+    }
     server->add(this, seat_.get());
     ic_->surroundingText().connect(
         [this](const char *text, uint32_t cursor, uint32_t anchor) {
@@ -145,6 +169,7 @@ WaylandIMInputContextV2::WaylandIMInputContextV2(
         if (pendingDeactivate_) {
             pendingDeactivate_ = false;
             keyboardGrab_.reset();
+            repeatInfo_.reset();
             // This is the only place we update wayland xkb mask, so it is ok to
             // reset it to 0. This breaks the caps lock or num lock. But we have
             // no other option until we can listen to the mod change globally.
@@ -162,13 +187,23 @@ WaylandIMInputContextV2::WaylandIMInputContextV2(
                                     Key(FcitxKey_None, KeyStates(), vkkey + 8),
                                     WL_KEYBOARD_KEY_STATE_RELEASED);
                     }
-                    vk_->modifiers(0, 0, 0, 0);
                 }
                 focusOutWrapper();
             }
+            if (!server_->parent()->persistentVirtualKeyboard()) {
+                vk_.reset();
+                vkReady_ = false;
+            }
         }
         if (pendingActivate_) {
             pendingActivate_ = false;
+            if (!server_->parent()->persistentVirtualKeyboard()) {
+                vk_.reset();
+                vkReady_ = false;
+                vk_.reset(
+                    server_->virtualKeyboardManagerV1()->createVirtualKeyboard(
+                        seat_.get()));
+            }
             // There can be only one grab. Always release old grab first.
             // It is possible when switching between two client, there will be
             // two activate. In that case we will have already one grab. The
@@ -198,7 +233,6 @@ WaylandIMInputContextV2::WaylandIMInputContextV2(
                     [this](int32_t rate, int32_t delay) {
                         repeatInfoCallback(rate, delay);
                     });
-                repeatInfoCallback(repeatRate_, repeatDelay_);
                 focusInWrapper();
             }
         }
@@ -212,7 +246,7 @@ WaylandIMInputContextV2::WaylandIMInputContextV2(
     });
     ic_->unavailable().connect([]() { WAYLANDIM_DEBUG() << "UNAVAILABLE"; });
     timeEvent_ = server_->instance()->eventLoop().addTimeEvent(
-        CLOCK_MONOTONIC, now(CLOCK_MONOTONIC), 0,
+        CLOCK_MONOTONIC, now(CLOCK_MONOTONIC), 1,
         [this](EventSourceTime *, uint64_t) {
             repeat();
             return true;
@@ -245,7 +279,7 @@ void WaylandIMInputContextV2::repeat() {
     if (!ic->keyEvent(event)) {
         sendKeyToVK(repeatTime_, event.rawKey(), WL_KEYBOARD_KEY_STATE_PRESSED);
     }
-    uint64_t interval = 1000000 / repeatRate_;
+    uint64_t interval = 1000000 / repeatRate();
     timeEvent_->setTime(timeEvent_->time() + interval);
     timeEvent_->setOneShot();
 }
@@ -419,9 +453,11 @@ void WaylandIMInputContextV2::keymapCallback(uint32_t format, int32_t fd,
     server_->stateMask_.mod5_mask =
         1 << xkb_keymap_mod_get_index(server_->keymap_.get(), "Mod5");
 
-    if (keymapChanged) {
-        vk_->keymap(format, scopeFD.fd(), size);
-        vkReady_ = true;
+    if (keymapChanged || !vkReady_) {
+        if (vk_) {
+            vk_->keymap(format, scopeFD.fd(), size);
+            vkReady_ = true;
+        }
     }
 
     server_->parent_->wayland()->call<IWaylandModule::reloadXkbOption>();
@@ -453,13 +489,13 @@ void WaylandIMInputContextV2::keyCallback(uint32_t serial, uint32_t time,
         timeEvent_->setEnabled(false);
     } else if (state == WL_KEYBOARD_KEY_STATE_PRESSED &&
                xkb_keymap_key_repeats(server_->keymap_.get(), code)) {
-        if (repeatRate_) {
+        if (repeatRate() > 0) {
             repeatKey_ = key;
             repeatTime_ = time;
             repeatSym_ = event.rawKey().sym();
             // Let's trick the key event system by fake our first.
             // Remove 100 from the initial interval.
-            timeEvent_->setNextInterval(repeatDelay_ * 1000 - repeatHackDelay);
+            timeEvent_->setNextInterval(repeatDelay() * 1000 - repeatHackDelay);
             timeEvent_->setOneShot();
         }
     }
@@ -481,7 +517,7 @@ void WaylandIMInputContextV2::keyCallback(uint32_t serial, uint32_t time,
         WAYLANDIM_DEBUG() << "Engine handling speed can not keep up with key "
                              "repetition rate.";
         timeEvent_->setNextInterval(
-            std::min(1000, repeatDelay_ * 1000 - repeatHackDelay));
+            std::clamp(repeatDelay() * 1000 - repeatHackDelay, 0, 1000));
     }
 }
 void WaylandIMInputContextV2::modifiersCallback(uint32_t /*serial*/,
@@ -500,8 +536,9 @@ void WaylandIMInputContextV2::modifiersCallback(uint32_t /*serial*/,
     server_->instance()->updateXkbStateMask(
         server_->group()->display(), mods_depressed, mods_latched, mods_locked);
     mask = xkb_state_serialize_mods(
-        server_->state_.get(), static_cast<xkb_state_component>(
-                                   XKB_STATE_DEPRESSED | XKB_STATE_LATCHED));
+        server_->state_.get(),
+        static_cast<xkb_state_component>(XKB_STATE_MODS_DEPRESSED |
+                                         XKB_STATE_MODS_LATCHED));
 
     server_->modifiers_ = 0;
     if (mask & server_->stateMask_.shift_mask) {
@@ -535,8 +572,7 @@ void WaylandIMInputContextV2::modifiersCallback(uint32_t /*serial*/,
 }
 
 void WaylandIMInputContextV2::repeatInfoCallback(int32_t rate, int32_t delay) {
-    repeatRate_ = rate;
-    repeatDelay_ = delay;
+    repeatInfo_ = std::make_tuple(rate, delay);
 }
 
 void WaylandIMInputContextV2::sendKeyToVK(uint32_t time, const Key &key,
@@ -553,7 +589,7 @@ void WaylandIMInputContextV2::sendKeyToVK(uint32_t time, const Key &key,
     // Erase old to ensure order, and released ones can the be removed.
     pressedVKKey_.erase(code);
     if (state == WL_KEYBOARD_KEY_STATE_PRESSED &&
-        xkb_keymap_key_repeats(server_->keymap_.get(), code)) {
+        xkb_keymap_key_repeats(server_->keymap_.get(), key.code())) {
         pressedVKKey_[code] = time;
     }
     vk_->key(time, code, state);
@@ -668,4 +704,13 @@ void WaylandIMInputContextV2::deleteSurroundingTextDelegate(
                                startBytes + sizeBytes - cursorBytes);
     ic_->commit(serial_);
 }
+
+int32_t WaylandIMInputContextV2::repeatRate() const {
+    return server_->repeatRate(seat_, repeatInfo_);
+}
+
+int32_t WaylandIMInputContextV2::repeatDelay() const {
+    return server_->repeatDelay(seat_, repeatInfo_);
+}
+
 } // namespace fcitx
