@@ -8,8 +8,22 @@
 #define _FCITX5_FRONTEND_WAYLANDIM_WAYLANDIMSERVERV2_H_
 
 #include <cstdint>
-#include "fcitx-utils/event.h"
+#include <memory>
+#include <optional>
+#include <string>
+#include <tuple>
+#include <unordered_map>
+#include <vector>
+#include "fcitx-utils/eventloopinterface.h"
+#include "fcitx-utils/key.h"
+#include "fcitx-utils/keysymgen.h"
 #include "fcitx-utils/misc_p.h"
+#include "fcitx-utils/signals.h"
+#include "fcitx/event.h"
+#include "fcitx/focusgroup.h"
+#include "fcitx/inputcontext.h"
+#include "fcitx/inputcontextmanager.h"
+#include "fcitx/instance.h"
 #include "virtualinputcontext.h"
 #include "waylandimserverbase.h"
 #include "zwp_input_method_keyboard_grab_v2.h"
@@ -41,6 +55,7 @@ public:
     FocusGroup *group() { return group_; }
     auto *xkbState() { return state_.get(); }
     auto *inputMethodManagerV2() { return inputMethodManagerV2_.get(); }
+    auto *virtualKeyboardManagerV1() { return virtualKeyboardManagerV1_.get(); }
 
     bool hasKeyboardGrab() const;
 
@@ -72,8 +87,7 @@ class WaylandIMInputContextV2 : public VirtualInputContextGlue {
 public:
     WaylandIMInputContextV2(InputContextManager &inputContextManager,
                             WaylandIMServerV2 *server,
-                            std::shared_ptr<wayland::WlSeat> seat,
-                            wayland::ZwpVirtualKeyboardV1 *vk);
+                            std::shared_ptr<wayland::WlSeat> seat);
     ~WaylandIMInputContextV2() override;
 
     const char *frontend() const override { return "wayland_v2"; }
@@ -115,6 +129,9 @@ private:
     void repeatInfoCallback(int32_t rate, int32_t delay);
     void sendKeyToVK(uint32_t time, const Key &key, uint32_t state) const;
 
+    int32_t repeatRate() const;
+    int32_t repeatDelay() const;
+
     WaylandIMServerV2 *server_;
     std::shared_ptr<wayland::WlSeat> seat_;
     std::unique_ptr<wayland::ZwpInputMethodV2> ic_;
@@ -134,7 +151,7 @@ private:
     uint32_t repeatTime_ = 0;
     KeySym repeatSym_ = FcitxKey_None;
 
-    int32_t repeatRate_ = 40, repeatDelay_ = 400;
+    std::optional<std::tuple<int32_t, int32_t>> repeatInfo_;
 
     mutable OrderedMap<uint32_t, uint32_t> pressedVKKey_;
 };
