@@ -5,9 +5,15 @@
  *
  */
 #include "wlrappmonitor.h"
-#include <wayland-client-core.h>
-#include "fcitx/addoninstance.h"
-#include "wayland_public.h"
+#include <cstddef>
+#include <cstdint>
+#include <list>
+#include <memory>
+#include <optional>
+#include <string>
+#include <unordered_map>
+#include "fcitx-utils/signals.h"
+#include "display.h"
 #include "zwlr_foreign_toplevel_handle_v1.h"
 #include "zwlr_foreign_toplevel_manager_v1.h"
 
