@@ -7,18 +7,32 @@
 #ifndef _FCITX_IM_KEYBOARD_KEYBOARD_H_
 #define _FCITX_IM_KEYBOARD_KEYBOARD_H_
 
+#include <functional>
+#include <memory>
+#include <string>
+#include <string_view>
+#include <tuple>
+#include <unordered_map>
+#include <unordered_set>
+#include <vector>
 #include "fcitx-config/configuration.h"
+#include "fcitx-config/enum.h"
 #include "fcitx-config/iniparser.h"
+#include "fcitx-config/option.h"
+#include "fcitx-config/rawconfig.h"
 #include "fcitx-utils/event.h"
+#include "fcitx-utils/handlertable.h"
 #include "fcitx-utils/i18n.h"
 #include "fcitx-utils/inputbuffer.h"
+#include "fcitx-utils/key.h"
 #include "fcitx/addonfactory.h"
+#include "fcitx/addoninstance.h"
 #include "fcitx/addonmanager.h"
+#include "fcitx/event.h"
 #include "fcitx/inputcontextproperty.h"
 #include "fcitx/inputmethodengine.h"
 #include "fcitx/instance.h"
 #include "compose.h"
-#include "isocodes.h"
 #include "keyboard_public.h"
 #include "longpress.h"
 #include "quickphrase_public.h"
@@ -76,16 +90,18 @@ FCITX_CONFIGURATION(
            "matching sequence.")}};
     SubConfigOption spell{this, "Spell", _("Spell"),
                           "fcitx://config/addon/spell"};
-    Option<bool> enableLongPress{this, "EnableLongPress",
-                                 _("Type special characters with long press"),
-                                 false};
-    Option<std::vector<std::string>> blocklistApplicationForLongPress{
-        this,
-        "LongPressBlocklist",
-        _("Applications disabled for long press"),
-        {"konsole"}};
-    SubConfigOption longPress{this, "LongPress", _("Long Press behavior"),
-                              "fcitx://config/addon/keyboard/longpress"};);
+    ConditionalHidden<isApple(), Option<bool>> enableLongPress{
+        this, "EnableLongPress", _("Type special characters with long press"),
+        false};
+    ConditionalHidden<isApple(), Option<std::vector<std::string>>>
+        blocklistApplicationForLongPress{
+            this,
+            "LongPressBlocklist",
+            _("Applications disabled for long press"),
+            {"konsole", "org.kde.konsole"}};
+    ConditionalHidden<isApple(), SubConfigOption> longPress{
+        this, "LongPress", _("Long Press behavior"),
+        "fcitx://config/addon/keyboard/longpress"};);
 
 class KeyboardEngine;
 
@@ -112,7 +128,7 @@ struct KeyboardEngineState : public InputContextProperty {
     bool handleLongPress(const KeyEvent &event);
     bool handleSpellModeTrigger(const InputMethodEntry &entry,
                                 const KeyEvent &event);
-    bool handleCandidateSelection(const KeyEvent &event);
+    bool handleCandidateSelection(const KeyEvent &event) const;
     std::tuple<std::string, bool> handleCompose(const KeyEvent &event);
     bool handleBackspace(const InputMethodEntry &entry);
 
@@ -125,7 +141,7 @@ struct KeyboardEngineState : public InputContextProperty {
 
     void updateCandidate(const InputMethodEntry &entry);
     // Update preedit and send ui update.
-    void setPreedit();
+    void setPreedit() const;
 
     // Return true if chr is pushed to buffer.
     // Return false if chr will be skipped by buffer, usually this means caller
@@ -152,7 +168,8 @@ public:
 
     const Configuration *getSubConfig(const std::string &path) const override;
 
-    void setSubConfig(const std::string &, const fcitx::RawConfig &) override;
+    void setSubConfig(const std::string & /*unused*/,
+                      const fcitx::RawConfig & /*unused*/) override;
 
     void reset(const InputMethodEntry &entry,
                InputContextEvent &event) override;
