@@ -7,11 +7,14 @@
 #ifndef _FCITX_CONFIG_CONFIGURATION_H_
 #define _FCITX_CONFIG_CONFIGURATION_H_
 
-#include <fcitx-config/option.h>
-#include <fcitx-utils/macros.h>
-#include "fcitxconfig_export.h"
-
 #include <memory>
+#include <string>
+#include <vector>
+#include <fcitx-config/fcitxconfig_export.h>
+#include <fcitx-config/option.h>
+#include <fcitx-config/optiontypename.h>
+#include <fcitx-config/rawconfig.h>
+#include <fcitx-utils/macros.h>
 
 #define FCITX_CONFIGURATION_EXTEND(NAME, SUBCLASS, ...)                        \
     class NAME;                                                                \
