@@ -5,10 +5,17 @@
  *
  */
 #include "eventdispatcher.h"
+#include <cstdint>
+#include <functional>
+#include <memory>
 #include <mutex>
 #include <queue>
 #include <stdexcept>
+#include <utility>
 #include "event.h"
+#include "eventloopinterface.h"
+#include "fs.h"
+#include "macros.h"
 #include "misc_p.h"
 #include "unixfd.h"
 
