@@ -12,12 +12,13 @@
 /// \file
 /// \brief Class to handler dynamic library.
 
+#include <cstddef>
 #include <functional>
 #include <memory>
 #include <string>
+#include <fcitx-utils/fcitxutils_export.h>
 #include <fcitx-utils/flags.h>
 #include <fcitx-utils/macros.h>
-#include "fcitxutils_export.h"
 
 namespace fcitx {
 
@@ -44,6 +45,7 @@ public:
     bool findData(const char *slug, const char *magic, size_t lenOfMagic,
                   const std::function<void(const char *data)> &parser);
     std::string error();
+    const std::string &path() const;
 
     template <typename Func>
     static auto toFunction(void *ptr) {
