@@ -12,23 +12,26 @@
 /// \file
 /// \brief Log utilities.
 
-#include <cstdlib>
 #include <iostream>
 #include <list>
 #include <map>
 #include <memory>
+#include <optional>
 #include <set>
 #include <string>
 #include <tuple>
 #include <type_traits>
 #include <unordered_map>
 #include <unordered_set>
+#include <utility>
+#include <vector>
+#include <fcitx-utils/fcitxutils_export.h>
 #include <fcitx-utils/fs.h>
 #include <fcitx-utils/key.h>
+#include <fcitx-utils/macros.h>
 #include <fcitx-utils/metastring.h>
 #include <fcitx-utils/misc.h>
 #include <fcitx-utils/tuplehelpers.h>
-#include "fcitxutils_export.h"
 
 namespace fcitx {
 
@@ -140,7 +143,7 @@ public:
 
     template <typename T>
     inline LogMessageBuilder &operator<<(const std::optional<T> &opt) {
-        *this << "optional(has_value=" << opt.has_value();
+        *this << "optional(has_value=" << opt.has_value() << " ";
         if (opt.has_value()) {
             *this << *opt;
         }
@@ -266,7 +269,8 @@ private:
     }
 
     template <typename... Args, int... S>
-    void printWithIndices(Sequence<S...>, const std::tuple<Args...> &tuple) {
+    void printWithIndices(Sequence<S...> /*unused*/,
+                          const std::tuple<Args...> &tuple) {
         using swallow = int[];
         (void)swallow{
             0,
@@ -316,7 +320,7 @@ private:
     FCITX_LOGC_IF(::fcitx::Log::defaultCategory, LEVEL, CONDITION)
 
 #define FCITX_ASSERT(...)                                                      \
-    FCITX_LOG_IF(Fatal, !(__VA_ARGS__)) << #__VA_ARGS__ << " failed."
+    FCITX_LOG_IF(Fatal, !(__VA_ARGS__)) << #__VA_ARGS__ << " failed. "
 
 #define FCITX_DEFINE_LOG_CATEGORY(name, ...)                                   \
     const ::fcitx::LogCategory &name() {                                       \
