@@ -8,15 +8,16 @@
 #define _FCITX_UTILS_MISC_H_
 
 #include <unistd.h>
-#include <cstdint>
+#include <cstddef>
 #include <cstdio>
 #include <cstdlib>
+#include <functional>
 #include <memory>
 #include <string>
-#include <utility>
+#include <type_traits>
 #include <vector>
+#include <fcitx-utils/fcitxutils_export.h>
 #include <fcitx-utils/macros.h>
-#include "fcitxutils_export.h"
 
 namespace fcitx {
 
@@ -49,11 +50,11 @@ inline const Parent *parentFromMember(const Member *member,
 template <typename Iter>
 class KeyIterator {
 public:
-    typedef typename Iter::iterator_category iterator_category;
-    typedef typename Iter::value_type::first_type value_type;
-    typedef typename Iter::difference_type difference_type;
-    typedef typename Iter::value_type::first_type &reference;
-    typedef typename Iter::value_type::first_type *pointer;
+    using iterator_category = typename Iter::iterator_category;
+    using value_type = typename Iter::value_type::first_type;
+    using difference_type = typename Iter::difference_type;
+    using reference = typename Iter::value_type::first_type &;
+    using pointer = typename Iter::value_type::first_type *;
 
     KeyIterator(Iter iter) : iter_(iter) {}
     FCITX_INLINE_DEFINE_DEFAULT_DTOR_AND_COPY(KeyIterator)
@@ -120,8 +121,9 @@ struct FunctionDeleter {
 };
 template <typename T, auto FreeFunction = std::free>
 using UniqueCPtr = std::unique_ptr<T, FunctionDeleter<FreeFunction>>;
-static_assert(sizeof(char *) == sizeof(UniqueCPtr<char>),
-              ""); // ensure no overhead
+static_assert(
+    sizeof(char *) == sizeof(UniqueCPtr<char>),
+    "UniqueCPtr size is not same as raw pointer."); // ensure no overhead
 
 using UniqueFilePtr = std::unique_ptr<FILE, FunctionDeleter<std::fclose>>;
 
@@ -169,6 +171,19 @@ FCITXUTILS_EXPORT constexpr inline bool isApple() {
 #endif
 }
 
+/**
+ * Util function that returns whether it is compile against emscripten.
+ *
+ * @since 5.1.12
+ */
+FCITXUTILS_EXPORT constexpr inline bool isEmscripten() {
+#if defined(__EMSCRIPTEN__)
+    return true;
+#else
+    return false;
+#endif
+}
+
 } // namespace fcitx
 
 #endif // _FCITX_UTILS_MISC_H_
