@@ -10,14 +10,17 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <algorithm>
+#include <cstdint>
 #include <cstdlib>
 #include <cstring>
+#include <functional>
+#include <initializer_list>
 #include <list>
 #include <string>
-#include <type_traits>
 #include <unordered_map>
-#include <config.h>
-#include "endian_p.h"
+#include <utility>
+#include <fcitx-utils/endian_p.h>
+#include "config.h"
 
 namespace fcitx {
 
@@ -51,6 +54,7 @@ public:
     bool empty() const { return order_.empty(); }
 
     const T &front() const { return order_.front(); }
+    T &front() { return order_.front(); }
 
     void clear() {
         dict_.clear();
