@@ -6,13 +6,14 @@
  */
 
 #include "utf8.h"
+#include <cstdint>
+#include <string>
 #include "cutf8.h"
 
 namespace fcitx::utf8 {
 
 bool UCS4IsValid(uint32_t code) {
-    return ((code) < 0x110000 && (((code)&0xFFFFF800) != 0xD800) &&
-            ((code) < 0xFDD0 || (code) > 0xFDEF) && ((code)&0xFFFE) != 0xFFFE);
+    return ((code) < 0x110000 && (((code) & 0xFFFFF800) != 0xD800));
 }
 
 std::string UCS4ToUTF8(uint32_t code) {
