@@ -5,7 +5,9 @@ const struct wl_buffer_listener WlBuffer::listener = {
     [](void *data, wl_buffer *wldata) {
         auto *obj = static_cast<WlBuffer *>(data);
         assert(*obj == wldata);
-        { return obj->release()(); }
+        {
+            return obj->release()();
+        }
     },
 };
 WlBuffer::WlBuffer(wl_buffer *data)
