@@ -6,12 +6,16 @@ const struct wl_registry_listener WlRegistry::listener = {
        uint32_t version) {
         auto *obj = static_cast<WlRegistry *>(data);
         assert(*obj == wldata);
-        { return obj->global()(name, interface, version); }
+        {
+            return obj->global()(name, interface, version);
+        }
     },
     [](void *data, wl_registry *wldata, uint32_t name) {
         auto *obj = static_cast<WlRegistry *>(data);
         assert(*obj == wldata);
-        { return obj->globalRemove()(name); }
+        {
+            return obj->globalRemove()(name);
+        }
     },
 };
 WlRegistry::WlRegistry(wl_registry *data)
@@ -20,6 +24,8 @@ WlRegistry::WlRegistry(wl_registry *data)
     wl_registry_add_listener(*this, &WlRegistry::listener, this);
 }
 void WlRegistry::destructor(wl_registry *data) {
-    { return wl_registry_destroy(data); }
+    {
+        return wl_registry_destroy(data);
+    }
 }
 } // namespace fcitx::wayland
