@@ -7,15 +7,28 @@
 
 #include "addonmanager.h"
 #include <fcntl.h>
+#include <cstdint>
+#include <memory>
+#include <stdexcept>
+#include <string>
 #include <unordered_map>
 #include <unordered_set>
+#include <utility>
+#include <vector>
 #include "fcitx-config/iniparser.h"
+#include "fcitx-config/rawconfig.h"
 #include "fcitx-utils/log.h"
+#include "fcitx-utils/macros.h"
 #include "fcitx-utils/misc_p.h"
-#include "fcitx/addoninstance.h"
+#include "fcitx-utils/semver.h"
+#include "fcitx-utils/standardpath.h"
+#include "fcitx-utils/unixfd.h"
+#include "addoninfo.h"
+#include "addoninstance.h"
 #include "addoninstance_p.h"
 #include "addonloader.h"
 #include "addonloader_p.h"
+#include "config.h"
 #include "instance.h"
 
 namespace fcitx {
@@ -60,7 +73,7 @@ enum class DependencyCheckStatus {
 
 class AddonManagerPrivate {
 public:
-    AddonManagerPrivate() : instance_(nullptr) {}
+    AddonManagerPrivate() {}
 
     Addon *addon(const std::string &name) const {
         auto iter = addons_.find(name);
@@ -76,9 +89,8 @@ public:
             if (dependency == "core") {
                 if (depVersion <= version_) {
                     continue;
-                } else {
-                    return DependencyCheckStatus::Failed;
                 }
+                return DependencyCheckStatus::Failed;
             }
             Addon *dep = addon(dependency);
             if (!dep || !dep->isLoadable()) {
@@ -250,12 +262,11 @@ void AddonManager::load(const std::unordered_set<std::string> &enabled,
     const auto &path = StandardPath::global();
     d->timestamp_ =
         path.timestamp(StandardPath::Type::PkgData, d->addonConfigDir_);
-    auto fileMap =
-        path.multiOpen(StandardPath::Type::PkgData, d->addonConfigDir_,
-                       O_RDONLY, filter::Suffix(".conf"));
+    auto fileNames = path.locate(StandardPath::Type::PkgData,
+                                 d->addonConfigDir_, filter::Suffix(".conf"));
     bool enableAll = enabled.count("all");
     bool disableAll = disabled.count("all");
-    for (const auto &[fileName, file] : fileMap) {
+    for (const auto &[fileName, fullName] : fileNames) {
         // remove .conf
         std::string name = fileName.substr(0, fileName.size() - 5);
         if (name == "core") {
@@ -267,7 +278,8 @@ void AddonManager::load(const std::unordered_set<std::string> &enabled,
         }
 
         RawConfig config;
-        readFromIni(config, file.fd());
+        UnixFD fd = UnixFD::own(open(fullName.c_str(), O_RDONLY));
+        readFromIni(config, fd.fd());
 
         // override configuration
         auto addon = std::make_unique<Addon>(name, config);
@@ -403,7 +415,7 @@ void AddonManager::setAddonOptions(
 
 std::vector<std::string> AddonManager::addonOptions(const std::string &name) {
     FCITX_D();
-    if (auto options = findValue(d->options_, name)) {
+    if (auto *options = findValue(d->options_, name)) {
         return *options;
     }
     return {};
