@@ -10,8 +10,8 @@
 #include <cstdint>
 #include <fcitx-utils/capabilityflags.h>
 #include <fcitx-utils/key.h>
+#include <fcitx/fcitxcore_export.h>
 #include <fcitx/userinterface.h>
-#include "fcitxcore_export.h"
 
 /// \addtogroup FcitxCore
 /// \{
@@ -140,6 +140,8 @@ enum class EventType : uint32_t {
     InputContextDeleteSurroundingText = InputMethodEventFlag | 0x3,
     InputContextUpdatePreedit = InputMethodEventFlag | 0x4,
     InputContextUpdateUI = InputMethodEventFlag | 0x5,
+    InputContextCommitStringWithCursor = InputMethodEventFlag | 0x6,
+    InputContextFlushUI = InputMethodEventFlag | 0x7,
 
     /**
      * This is generated when input method group changed.
@@ -332,6 +334,14 @@ public:
      */
     bool forward() const { return forward_; }
 
+    /**
+     * Whether this key event is derived from a virtual keyboard
+     *
+     * @return bool
+     * @since 5.1.2
+     */
+    bool isVirtual() const { return origKey_.isVirtual(); }
+
 protected:
     Key key_, origKey_, rawKey_;
     bool isRelease_;
@@ -409,6 +419,28 @@ protected:
     std::string text_;
 };
 
+/**
+ * Event for commit string with cursor
+ *
+ * @see InputContext::commitStringWithCursor
+ * @since 5.1.2
+ */
+class FCITXCORE_EXPORT CommitStringWithCursorEvent : public InputContextEvent {
+public:
+    CommitStringWithCursorEvent(std::string text, size_t cursor,
+                                InputContext *context)
+        : InputContextEvent(context,
+                            EventType::InputContextCommitStringWithCursor),
+          text_(std::move(text)), cursor_(cursor) {}
+
+    const std::string &text() const { return text_; }
+    size_t cursor() const { return cursor_; }
+
+protected:
+    std::string text_;
+    size_t cursor_;
+};
+
 class FCITXCORE_EXPORT InvokeActionEvent : public InputContextEvent {
 public:
     enum class Action { LeftClick, RightClick };
@@ -478,6 +510,24 @@ protected:
     bool immediate_;
 };
 
+/**
+ * Events triggered that user interface manager that flush the UI update.
+ *
+ * @since 5.1.2
+ */
+class FCITXCORE_EXPORT InputContextFlushUIEvent : public InputContextEvent {
+public:
+    InputContextFlushUIEvent(UserInterfaceComponent component,
+                             InputContext *context)
+        : InputContextEvent(context, EventType::InputContextFlushUI),
+          component_(component) {}
+
+    UserInterfaceComponent component() const { return component_; }
+
+protected:
+    UserInterfaceComponent component_;
+};
+
 class FCITXCORE_EXPORT VirtualKeyboardVisibilityChangedEvent : public Event {
 public:
     VirtualKeyboardVisibilityChangedEvent()
