@@ -10,7 +10,12 @@
 #include <signal.h>
 #include <sys/wait.h>
 #include <unistd.h>
+#include <cstdint>
+#include <ctime>
+#include <functional>
+#include <memory>
 #include <stdexcept>
+#include <string>
 #include <unordered_map>
 #include <utility>
 #include <fmt/format.h>
@@ -18,8 +23,12 @@
 #include "fcitx-config/iniparser.h"
 #include "fcitx-utils/capabilityflags.h"
 #include "fcitx-utils/event.h"
+#include "fcitx-utils/eventdispatcher.h"
+#include "fcitx-utils/eventloopinterface.h"
 #include "fcitx-utils/i18n.h"
+#include "fcitx-utils/key.h"
 #include "fcitx-utils/log.h"
+#include "fcitx-utils/macros.h"
 #include "fcitx-utils/misc.h"
 #include "fcitx-utils/standardpath.h"
 #include "fcitx-utils/stringutils.h"
@@ -574,6 +583,7 @@ void InputState::reset() {
     pendingGroupIndex_ = 0;
     keyReleased_ = -1;
     lastKeyPressed_ = Key();
+    lastKeyPressedTime_ = 0;
     totallyReleased_ = true;
 }
 
@@ -641,9 +651,10 @@ Instance::Instance(int argc, char **argv) {
     }
 
     // we need fork before this
-    d_ptr.reset(new InstancePrivate(this));
+    d_ptr = std::make_unique<InstancePrivate>(this);
     FCITX_D();
     d->arg_ = arg;
+    d->eventDispatcher_.attach(&d->eventLoop_);
     d->addonManager_.setInstance(this);
     d->addonManager_.setAddonOptions(arg.addonOptions_);
     d->icManager_.setInstance(this);
@@ -780,7 +791,12 @@ Instance::Instance(int argc, char **argv) {
                         origKey.isReleaseOfModifier(lastKeyPressed) &&
                         keyHandler.check()) {
                         if (isModifier) {
-                            keyHandler.trigger(inputState->totallyReleased_);
+                            if (d->globalConfig_.checkModifierOnlyKeyTimeout(
+                                    inputState->lastKeyPressedTime_)) {
+                                keyHandler.trigger(
+                                    inputState->totallyReleased_);
+                            }
+                            inputState->lastKeyPressedTime_ = 0;
                             if (origKey.hasModifier()) {
                                 inputState->totallyReleased_ = false;
                             }
@@ -813,6 +829,8 @@ Instance::Instance(int argc, char **argv) {
                         inputState->keyReleased_ = idx;
                         inputState->lastKeyPressed_ = origKey;
                         if (isModifier) {
+                            inputState->lastKeyPressedTime_ =
+                                now(CLOCK_MONOTONIC);
                             // don't forward to input method, but make it pass
                             // through to client.
                             return keyEvent.filter();
@@ -913,7 +931,8 @@ Instance::Instance(int argc, char **argv) {
                              << " rawKey: " << keyEvent.rawKey()
                              << " origKey: " << keyEvent.origKey()
                              << " Release:" << keyEvent.isRelease()
-                             << " keycode: " << keyEvent.origKey().code();
+                             << " keycode: " << keyEvent.origKey().code()
+                             << " program: " << ic->program();
 
             if (keyEvent.isRelease()) {
                 return;
@@ -971,6 +990,13 @@ Instance::Instance(int argc, char **argv) {
 #ifdef ENABLE_KEYBOARD
             if (keyEvent.forward()) {
                 FCITX_D();
+                // Always let the release key go through, since it shouldn't
+                // produce character. Otherwise it may wrongly trigger wayland
+                // client side repetition.
+                if (keyEvent.isRelease()) {
+                    keyEvent.filter();
+                    return;
+                }
                 auto *inputState = ic->propertyFor(&d->inputStateFactory_);
                 if (auto *xkbState = inputState->customXkbState()) {
                     if (auto utf32 = xkb_state_key_get_utf32(
@@ -986,10 +1012,8 @@ Instance::Instance(int argc, char **argv) {
                                 keyEvent.origKey().sym()) {
                             return;
                         }
-                        if (!keyEvent.isRelease()) {
-                            FCITX_KEYTRACE() << "Will commit char: " << utf32;
-                            ic->commitString(utf8::UCS4ToUTF8(utf32));
-                        }
+                        FCITX_KEYTRACE() << "Will commit char: " << utf32;
+                        ic->commitString(utf8::UCS4ToUTF8(utf32));
                         keyEvent.filterAndAccept();
                     } else if (!keyEvent.key().states().test(KeyState::Ctrl) &&
                                keyEvent.rawKey().sym() !=
@@ -1008,13 +1032,31 @@ Instance::Instance(int argc, char **argv) {
         EventType::InputContextFocusIn, EventWatcherPhase::ReservedFirst,
         [this, d](Event &event) {
             auto &icEvent = static_cast<InputContextEvent &>(event);
+            auto isSameProgram = [&icEvent, d]() {
+                // Check if they are same IC, or they are same program.
+                return (icEvent.inputContext() == d->lastUnFocusedIc_.get()) ||
+                       (!icEvent.inputContext()->program().empty() &&
+                        (icEvent.inputContext()->program() ==
+                         d->lastUnFocusedProgram_));
+            };
+
+            if (d->globalConfig_.resetStateWhenFocusIn() ==
+                    PropertyPropagatePolicy::All ||
+                (d->globalConfig_.resetStateWhenFocusIn() ==
+                     PropertyPropagatePolicy::Program &&
+                 !isSameProgram())) {
+                if (d->globalConfig_.activeByDefault()) {
+                    activate(icEvent.inputContext());
+                } else {
+                    deactivate(icEvent.inputContext());
+                }
+            }
+
             activateInputMethod(icEvent);
 
-            if (virtualKeyboardAutoShow()) {
-                auto *inputContext = icEvent.inputContext();
-                if (!inputContext->clientControlVirtualkeyboardShow()) {
-                    inputContext->showVirtualKeyboard();
-                }
+            auto *inputContext = icEvent.inputContext();
+            if (!inputContext->clientControlVirtualkeyboardShow()) {
+                inputContext->showVirtualKeyboard();
             }
 
             if (!d->globalConfig_.showInputMethodInformationWhenFocusIn() ||
@@ -1054,14 +1096,15 @@ Instance::Instance(int argc, char **argv) {
         }));
     d->eventWatchers_.emplace_back(d->watchEvent(
         EventType::InputContextFocusOut, EventWatcherPhase::InputMethod,
-        [this](Event &event) {
+        [this, d](Event &event) {
             auto &icEvent = static_cast<InputContextEvent &>(event);
+            d->lastUnFocusedProgram_ = icEvent.inputContext()->program();
+            d->lastUnFocusedIc_ = icEvent.inputContext()->watch();
             deactivateInputMethod(icEvent);
-            if (virtualKeyboardAutoHide()) {
-                auto *inputContext = icEvent.inputContext();
-                if (!inputContext->clientControlVirtualkeyboardHide()) {
-                    inputContext->hideVirtualKeyboard();
-                }
+
+            auto *inputContext = icEvent.inputContext();
+            if (!inputContext->clientControlVirtualkeyboardHide()) {
+                inputContext->hideVirtualKeyboard();
             }
         }));
     d->eventWatchers_.emplace_back(d->watchEvent(
@@ -1331,6 +1374,9 @@ void Instance::handleSignal() {
             exit();
         } else if (signo == SIGUSR1) {
             reloadConfig();
+        } else if (signo == SIGCHLD) {
+            d->zombieReaper_->setNextInterval(2000000);
+            d->zombieReaper_->setOneShot();
         }
     }
 }
@@ -1381,6 +1427,15 @@ void Instance::initialize() {
             }
             return false;
         });
+    d->zombieReaper_ = d->eventLoop_.addTimeEvent(
+        CLOCK_MONOTONIC, now(CLOCK_MONOTONIC), 0,
+        [](EventSourceTime *, uint64_t) {
+            pid_t res;
+            while ((res = waitpid(-1, nullptr, WNOHANG)) > 0) {
+            }
+            return false;
+        });
+    d->zombieReaper_->setEnabled(false);
 
     d->exitEvent_ = d->eventLoop_.addExitEvent([this](EventSource *) {
         FCITX_DEBUG() << "Running save...";
@@ -1497,6 +1552,11 @@ EventLoop &Instance::eventLoop() {
     return d->eventLoop_;
 }
 
+EventDispatcher &Instance::eventDispatcher() {
+    FCITX_D();
+    return d->eventDispatcher_;
+}
+
 InputContextManager &Instance::inputContextManager() {
     FCITX_D();
     return d->icManager_;
@@ -2440,6 +2500,14 @@ void Instance::showInputMethodInformation(InputContext *ic) {
     d->showInputMethodInformation(ic);
 }
 
+void Instance::showCustomInputMethodInformation(InputContext *ic,
+                                                const std::string &message) {
+    FCITX_DEBUG() << "Input method switched";
+    FCITX_D();
+    auto *inputState = ic->propertyFor(&d->inputStateFactory_);
+    inputState->showInputMethodInformation(message);
+}
+
 bool Instance::checkUpdate() const {
     FCITX_D();
     return (isInFlatpak() && fs::isreg("/app/.updated")) ||
