@@ -784,9 +784,11 @@ DBusModule::DBusModule(Instance *instance)
         });
 
     selfWatcher_ = serviceWatcher_->watchService(
-        FCITX_DBUS_SERVICE,
-        [uniqueName, instance](const std::string &, const std::string &,
-                               const std::string &newName) {
+        FCITX_DBUS_SERVICE, [uniqueName, instance](const std::string &service,
+                                                   const std::string &oldName,
+                                                   const std::string &newName) {
+            FCITX_INFO() << "Service name change: " << service << " " << oldName
+                         << " " << newName;
             if (newName != uniqueName) {
                 instance->exit();
             }
@@ -852,4 +854,4 @@ class DBusModuleFactory : public AddonFactory {
 };
 } // namespace fcitx
 
-FCITX_ADDON_FACTORY(fcitx::DBusModuleFactory)
+FCITX_ADDON_FACTORY_V2(dbus, fcitx::DBusModuleFactory)
