@@ -7,13 +7,10 @@
 #ifndef _FCITX_MODULES_UNICODE_UNICODE_H_
 #define _FCITX_MODULES_UNICODE_UNICODE_H_
 
-#include <map>
 #include "fcitx-config/configuration.h"
-#include "fcitx-config/enum.h"
 #include "fcitx-config/iniparser.h"
 #include "fcitx-utils/i18n.h"
 #include "fcitx-utils/key.h"
-#include "fcitx-utils/standardpath.h"
 #include "fcitx/addonfactory.h"
 #include "fcitx/addoninstance.h"
 #include "fcitx/addonmanager.h"
