@@ -625,7 +625,7 @@ void XCBUI::readXSettings() {
         if (!readCard16(&nameLen)) {
             return;
         }
-#define XSETTINGS_PAD(n, m) (((n) + (m)-1) & (~((m)-1)))
+#define XSETTINGS_PAD(n, m) (((n) + (m) - 1) & (~((m) - 1)))
         uint32_t namePad = XSETTINGS_PAD(nameLen, 4);
         if (std::distance(iter, data.cend()) < namePad) {
             return;
@@ -742,15 +742,15 @@ int XCBUI::scaledDPI(int dpi) {
     return targetDPI;
 }
 
-void XCBUI::resume() { updateTray(); }
+void XCBUI::resume() {}
 
 void XCBUI::suspend() {
     inputWindow_->update(nullptr);
-    updateTray();
+    trayWindow_->suspend();
 }
 
-void XCBUI::updateTray() {
-    bool enableTray = enableTray_ && !parent_->suspended();
+void XCBUI::setEnableTray(bool enable) {
+    bool enableTray = enable && !parent_->suspended();
     if (enableTray) {
         trayWindow_->resume();
     } else {
@@ -758,13 +758,6 @@ void XCBUI::updateTray() {
     }
 }
 
-void XCBUI::setEnableTray(bool enable) {
-    if (enable != enableTray_) {
-        enableTray_ = enable;
-        updateTray();
-    }
-}
-
 void XCBUI::scheduleUpdateScreen() {
     initScreenEvent_->setNextInterval(100000);
     initScreenEvent_->setOneShot();
