@@ -12,6 +12,8 @@
 #include "fcitx-utils/key.h"
 #include "fcitx-utils/log.h"
 #include "fcitx-utils/utf8.h"
+#include "fcitx/addonfactory.h"
+#include "fcitx/addoninstance.h"
 #include "fcitx/addonmanager.h"
 #include "fcitx/candidatelist.h"
 #include "fcitx/inputcontext.h"
@@ -32,7 +34,7 @@ public:
 
     void showVirtualKeyboard() { parent_->showVirtualKeyboardForcibly(); }
 
-    void hideVirtualKeyboard() { parent_->hideVirtualKeyboard(); }
+    void hideVirtualKeyboard() { parent_->hideVirtualKeyboardForcibly(); }
 
     void toggleVirtualKeyboard() { parent_->toggleVirtualKeyboard(); }
 
@@ -323,13 +325,25 @@ void VirtualKeyboard::showVirtualKeyboardForcibly() {
     showVirtualKeyboard();
 }
 
+void VirtualKeyboard::hideVirtualKeyboardForcibly() {
+    if (!available_) {
+        return;
+    }
+
+    hideVirtualKeyboard();
+
+    if (!instance_->virtualKeyboardAutoShow()) {
+        instance_->setInputMethodMode(InputMethodMode::PhysicalKeyboard);
+    }
+}
+
 void VirtualKeyboard::toggleVirtualKeyboard() {
     if (!available_) {
         return;
     }
 
     if (visible_) {
-        hideVirtualKeyboard();
+        hideVirtualKeyboardForcibly();
     } else {
         showVirtualKeyboardForcibly();
     }
@@ -420,7 +434,7 @@ std::vector<std::string> VirtualKeyboard::makeCandidateTextList(
         }
 
         auto candidateText =
-            instance_->outputFilter(inputContext, candidate.text());
+            instance_->outputFilter(inputContext, candidate.textWithComment());
         candidateTextList.push_back(candidateText.toString());
     }
 
@@ -446,7 +460,7 @@ std::vector<std::string> VirtualKeyboard::makeBulkCandidateTextList(
                 continue;
             }
 
-            candidateText = candidate.text();
+            candidateText = candidate.textWithComment();
         } catch (...) {
             break;
         }
@@ -459,13 +473,12 @@ std::vector<std::string> VirtualKeyboard::makeBulkCandidateTextList(
 
 int VirtualKeyboard::globalCursorIndex(
     std::shared_ptr<CandidateList> candidateList) const {
-    auto *commonCandidateList =
-        dynamic_cast<CommonCandidateList *>(candidateList.get());
-    if (commonCandidateList == nullptr) {
+    auto *bulkCursor = candidateList->toBulkCursor();
+    if (bulkCursor == nullptr) {
         return -1;
     }
 
-    return commonCandidateList->globalCursorIndex();
+    return bulkCursor->globalCursorIndex();
 }
 
 void VirtualKeyboard::updateCandidateArea(
@@ -536,4 +549,4 @@ public:
 };
 } // namespace fcitx
 
-FCITX_ADDON_FACTORY(fcitx::VirtualKeyboardFactory);
+FCITX_ADDON_FACTORY_V2(virtualkeyboard, fcitx::VirtualKeyboardFactory);
