@@ -34,9 +34,19 @@ int main() {
     FCITX_ASSERT(result);
     auto result2 = 8 == addon->call<fcitx::IDummyAddon::addOne>(7);
     FCITX_ASSERT(result2);
+
+    FCITX_ASSERT(!addon->call<fcitx::IDummyAddon::testCopy>().isCopy());
+
     auto *addon2 = manager.addon("dummyaddon2");
     FCITX_ASSERT(addon2);
     auto *addon3 = manager.addon("dummyaddon3");
     FCITX_ASSERT(!addon3);
+    // loading dummyaddondeps will change log rule level to 3.
+    FCITX_ASSERT(
+        !fcitx::Log::defaultCategory().checkLogLevel(fcitx::LogLevel::Info));
+    FCITX_ASSERT(
+        fcitx::Log::defaultCategory().checkLogLevel(fcitx::LogLevel::Warn));
+    fcitx::Log::setLogRule("default=5");
+
     return 0;
 }
