@@ -6,12 +6,11 @@
  */
 
 #include <libgen.h>
-#include <sys/stat.h>
 #include <unistd.h>
-#include <fcitx-utils/stringutils.h>
 #include "fcitx-utils/fs.h"
 #include "fcitx-utils/log.h"
 #include "fcitx-utils/misc.h"
+#include "fcitx-utils/stringutils.h"
 
 using namespace fcitx::fs;
 using namespace fcitx;
