@@ -6,19 +6,17 @@
  */
 
 #include <string.h>
+#include "fcitx-utils/fcitxutils_export.h"
 #include "fcitx-utils/library.h"
 #include "fcitx-utils/log.h"
-#include "fcitxutils_export.h"
 
 #define DATA "AAAAAAAAA"
 #define MAGIC "MAGIC_TEST_DATA"
 
 extern "C" {
-FCITXUTILS_EXPORT
-char magic_test[] = MAGIC DATA;
+FCITXUTILS_EXPORT char magic_test[] = MAGIC DATA;
 
-FCITXUTILS_EXPORT
-int func() { return 0; }
+FCITXUTILS_EXPORT int func() { return 0; }
 }
 
 void parser(const char *data) { FCITX_ASSERT(strcmp(data, DATA) == 0); }
